'use strict';

module.exports = function MetadataExtractor(rawFragmentShader) {
  // First pull out the comment JSON to get the metadata.
  // This regex (should) match quotes in the form /* */.
  var regex = /\*([^*]|[\r\n]|(\*+([^*/]|[\r\n])))*\*+/;
  var results = regex.exec(rawFragmentShader);

  if (!results) {
    throw new Error('There is no metadata here.');
  }

  var metadataString = results[0];
  metadataString = metadataString.substring(1, metadataString.length - 1);
  var metadata = void 0;
  try {
    metadata = JSON.parse(metadataString);
  } catch (e) {
    var tokens = e.message.split(' ');
    var position = parseInt(tokens[tokens.length - 1], 10);
    var lineNumber = 1;
    for (var i = 0; i < position; i++) {
      if (metadataString[i] === '\n') {
        lineNumber++;
      }
    }
    var lineError = new Error('Something is wrong with your metadata at position ' + position + ' and line ' + lineNumber);
    lineError.lineNumber = lineNumber;
    throw lineError;
  }

  var startIndex = rawFragmentShader.indexOf('/*');
  var endIndex = rawFragmentShader.indexOf('*/');
  return {
    objectValue: metadata,
    stringValue: metadataString,
    startIndex: startIndex,
    endIndex: endIndex
  };
};